//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  Harmonica Tabs Plugin
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.2
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import MuseScore 1.0

MuseScore {
   version: "2.0"
   description: "Harmonica Tab plugin"
   menuPath: "Plugins.Harmonica Tablature"
    pluginType: "dialog"
    
    id: window
      width:220
      height: 130
      ExclusiveGroup { id: exclusiveGroupKey }
      ColumnLayout {
        id: column
        anchors.margins : 10
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: 60
        CheckBox {
          id: keyC_CheckBox
          text: "key C "
          checked: true
          exclusiveGroup: exclusiveGroupKey
        }
        CheckBox {
          id: keyA_CheckBox
          text: "key A "
          exclusiveGroup: exclusiveGroupKey
        }
        CheckBox {
          id: keyG_CheckBox
          text: "key G "
          exclusiveGroup: exclusiveGroupKey
        }
      }
      RowLayout {
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: column.bottom
            height: 70
            Button {
              id: okButton
              text: "Ok"
              onClicked: {
                apply()
                Qt.quit()
              }
            }
            Button {
              id: closeButton
              text: "Close"
              onClicked: { Qt.quit() }
            }

        }
    
        function tabNotes(notes, text) {

         var richter = ["+1",  "-1b",  "-1", "+1o", "+2",  "-2bb",   "-2b",  "-2",   "-3bbb", "-3bb",  "-3b",   "-3",
            "+4",   "-4b",  "-4", "+4o", "+5",  "-5",     "+5o",  "+6",   "-6b",   "-6",    "+6o",   "-7",
            "+7",   "-7o",  "-8", "+8b", "+8",  "-9",     "+9b",  "+9",   "-9o",  "-10",   "+10bb",  "+10b",
            "+10" ];    //Standard Richter tuning with overbends
      
        var tuning = richter
        var harpkey = 60 // C4 (MIDI pitch value)
        if (keyC_CheckBox.checked)
            harpkey = 60
        else if (keyA_CheckBox.checked)
            harpkey = 57 //  A3 
        else if (keyG_CheckBox.checked)
            harpkey =  55 // G3
        
      for (var i = 0; i < notes.length; i++) {
         var sep = "\n"; // change to "," if you want them horizontally
         if ( i > 0 )
            text.text = sep + text.text;

         if (typeof notes[i].pitch === "undefined") // just in case
            return
         if (typeof tuning[notes[i].pitch - harpkey] === "undefined")
            text.text = "X";
         else
            text.text = tuning[notes[i].pitch - harpkey] + text.text;
             }
      }

   function applyToSelection(func) {
      if (typeof curScore === 'undefined')
        Qt.quit();
      var cursor = curScore.newCursor();
      var startStaff;
      var endStaff;
      var endTick;
      var fullScore = false;
      cursor.rewind(1);
      if (!cursor.segment) { // no selection
        fullScore = true;
        startStaff = 0; // start with 1st staff
        endStaff  = curScore.nstaves - 1; // and end with last
      } else {
        startStaff = cursor.staffIdx;
        cursor.rewind(2);
        if (cursor.tick == 0) {
          // this happens when the selection includes
          // the last measure of the score.
          // rewind(2) goes behind the last segment (where
          // there's none) and sets tick=0
          endTick = curScore.lastSegment.tick + 1;
        } else {
          endTick = cursor.tick;
        }
        endStaff   = cursor.staffIdx;
      }
      console.log(startStaff + " - " + endStaff + " - " + endTick)

      for (var staff = startStaff; staff <= endStaff; staff++) {
        for (var voice = 0; voice < 4; voice++) {
          cursor.rewind(1); // beginning of selection
          cursor.voice    = voice;
          cursor.staffIdx = staff;

          if (fullScore)  // no selection
            cursor.rewind(0); // beginning of score

          while (cursor.segment && (fullScore || cursor.tick < endTick)) {
            if (cursor.element && cursor.element.type == Element.CHORD) {
              var text = newElement(Element.STAFF_TEXT);

              var graceChords = cursor.element.graceNotes;
              for (var i = 0; i < graceChords.length; i++) {
                // iterate through all grace chords
                var notes = graceChords[i].notes;
                tabNotes(notes, text);
                // there seems to be no way of knowing the exact horizontal pos.
                // of a grace note, so we have to guess:
                text.pos.x = -2.5 * (graceChords.length - i);
                switch (voice) {
                  case 0: text.pos.y =  -1; break;
                  case 1: text.pos.y = 10; break;
                  case 2: text.pos.y = -2; break;
                  case 3: text.pos.y = 12; break;
                }

                cursor.add(text);
                // new text for next element
                text  = newElement(Element.STAFF_TEXT);
              }

              var notes = cursor.element.notes;
              tabNotes(notes, text);

              switch (voice) {
                case 0: text.pos.y =  -1; break;
                case 1: text.pos.y = 10; break;
                case 2: text.pos.y = -2; break;
                case 3: text.pos.y = 12; break;
              }
              if ((voice == 0) && (notes[0].pitch > 83))
                text.pos.x = 1;
              cursor.add(text);
            } // end if CHORD
            cursor.next();
          } // end while segment
          } // end for voice
          } // end for staff
        Qt.quit();
          } // end applyToSelection()
          
       function apply() {
        applyToSelection(tabNotes)            
      }         

onRun: {
      if (typeof curScore === 'undefined')
        Qt.quit();
   }
}
